@@ -24,7 +24,7 @@ TODOs:
 """
 
 __author__ = "Andy Buckley <andy.buckley@cern.ch"
-__version__ = "1.2.9"
+__version__ = "1.3.0"
 
 
 def _autotype(var):
@@ -161,16 +161,6 @@ class Particle(object):
 
 
 
-def readSLHAFile(spcfilename, **kwargs):
-    """
-    Read an SLHA file, returning dictionaries of blocks and decays.
-
-    Other keyword parameters are passed to readSLHA.
-    """
-    f = open(spcfilename, "r")
-    rtn = readSLHA(f.read(), kwargs)
-    f.close()
-    return rtn
 
 
 def readSLHA(spcstr, ignorenobr=False):
@@ -248,30 +238,6 @@ def readSLHA(spcstr, ignorenobr=False):
     return blocks, decays
 
 
-def readISAWIGFile(isafilename, **kwargs):
-    """
-    Read a spectrum definition from a file in the ISAWIG format, returning
-    dictionaries of blocks and decays. While this is not an SLHA format, it is
-    informally supported as a useful mechanism for converting ISAWIG spectra to
-    SLHA.
-
-    Other keyword parameters are passed to readSLHA.
-    """
-    f = open(isafilename, "r")
-    rtn = readISAWIG(f.read(), kwargs)
-    f.close()
-    return rtn
-
-
-def writeSLHAFile(spcfilename, blocks, decays, **kwargs):
-    """
-    Write an SLHA file from the supplied blocks and decays dicts.
-
-    Other keyword parameters are passed to writeSLHA.
-    """
-    f = open(spcfilename, "w")
-    f.write(writeSLHA(blocks, decays, kwargs))
-    f.close()
 
 
 # TODO: Split writeSLHA into writeSLHA{Blocks,Decays}
@@ -314,7 +280,7 @@ def writeSLHA(blocks, decays, ignorenobr=False, precision=8):
 
 
 ###############################################################################
-## ISAWIG handling
+## PDG <-> HERWIG particle ID code translations for ISAWIG handling
 
 ## Static array of HERWIG IDHW codes mapped to PDG MC ID codes, based on
 ## http://www.hep.phy.cam.ac.uk/~richardn/HERWIG/ISAWIG/susycodes.html
@@ -416,6 +382,110 @@ def herwigid2pdgid(hwid):
     return _HERWIGID2PDGID.get(hwid, hwid)
 
 
+## PDG MC ID codes mapped to HERWIG IDHW codes, based on
+## http://www.hep.phy.cam.ac.uk/~richardn/HERWIG/ISAWIG/susycodes.html
+## + the IDPDG array and section 4.13 of the HERWIG manual.
+_PDGID2HERWIGID = {}
+_PDGID2HERWIGID[      -1] = 7
+_PDGID2HERWIGID[      -2] = 8
+_PDGID2HERWIGID[      -3] = 9
+_PDGID2HERWIGID[      -4] = 10
+_PDGID2HERWIGID[      -5] = 11
+_PDGID2HERWIGID[      -6] = 12
+_PDGID2HERWIGID[      21] = 13
+_PDGID2HERWIGID[      22] = 59
+_PDGID2HERWIGID[      11] = 121
+_PDGID2HERWIGID[      12] = 122
+_PDGID2HERWIGID[      13] = 123
+_PDGID2HERWIGID[      14] = 124
+_PDGID2HERWIGID[      15] = 125
+_PDGID2HERWIGID[      16] = 126
+_PDGID2HERWIGID[     -11] = 127
+_PDGID2HERWIGID[     -12] = 128
+_PDGID2HERWIGID[     -13] = 129
+_PDGID2HERWIGID[     -14] = 130
+_PDGID2HERWIGID[     -15] = 131
+_PDGID2HERWIGID[     -16] = 132
+_PDGID2HERWIGID[      24] = 198 ## W+
+_PDGID2HERWIGID[     -24] = 199 ## W-
+_PDGID2HERWIGID[      23] = 200 ## Z
+_PDGID2HERWIGID[      25] = 203 ## HIGGSL0 (added for PDG standard -> HERWIG IDHW) # TODO: should be 201?
+_PDGID2HERWIGID[      26] = 203 ## HIGGSL0
+_PDGID2HERWIGID[      35] = 204 ## HIGGSH0
+_PDGID2HERWIGID[      36] = 205 ## HIGGSA0
+_PDGID2HERWIGID[      37] = 206 ## HIGGS+
+_PDGID2HERWIGID[     -37] = 207 ## HIGGS-
+_PDGID2HERWIGID[ 1000001] = 401 ## SSDLBR
+_PDGID2HERWIGID[-1000001] = 407 ## SSDLBR
+_PDGID2HERWIGID[ 1000002] = 402 ## SSULBR
+_PDGID2HERWIGID[-1000002] = 408 ## SSUL
+_PDGID2HERWIGID[ 1000003] = 403 ## SSSLBR
+_PDGID2HERWIGID[-1000003] = 409 ## SSSL
+_PDGID2HERWIGID[ 1000004] = 404 ## SSCLBR
+_PDGID2HERWIGID[-1000004] = 410 ## SSCL
+_PDGID2HERWIGID[ 1000005] = 405 ## SSB1BR
+_PDGID2HERWIGID[-1000005] = 411 ## SSB1
+_PDGID2HERWIGID[ 1000006] = 406 ## SST1BR
+_PDGID2HERWIGID[-1000006] = 412 ## SST1
+_PDGID2HERWIGID[ 2000001] = 413 ## SSDR
+_PDGID2HERWIGID[-2000001] = 419 ## SSDRBR
+_PDGID2HERWIGID[ 2000002] = 414 ## SSUR
+_PDGID2HERWIGID[-2000002] = 420 ## SSURBR
+_PDGID2HERWIGID[ 2000003] = 415 ## SSSR
+_PDGID2HERWIGID[-2000003] = 421 ## SSSRBR
+_PDGID2HERWIGID[ 2000004] = 416 ## SSCR
+_PDGID2HERWIGID[-2000004] = 422 ## SSCRBR
+_PDGID2HERWIGID[ 2000005] = 417 ## SSB2
+_PDGID2HERWIGID[-2000005] = 423 ## SSB2BR
+_PDGID2HERWIGID[ 2000006] = 418 ## SST2
+_PDGID2HERWIGID[-2000006] = 424 ## SST2BR
+_PDGID2HERWIGID[ 1000011] = 425 ## SSEL-
+_PDGID2HERWIGID[-1000011] = 431 ## SSEL+
+_PDGID2HERWIGID[ 1000012] = 426 ## SSNUEL
+_PDGID2HERWIGID[-1000012] = 432 ## SSNUELBR
+_PDGID2HERWIGID[ 1000013] = 427 ## SSMUL-
+_PDGID2HERWIGID[-1000013] = 433 ## SSMUL+
+_PDGID2HERWIGID[ 1000014] = 428 ## SSNUMUL
+_PDGID2HERWIGID[-1000014] = 434 ## SSNUMLBR
+_PDGID2HERWIGID[ 1000015] = 429 ## SSTAU1-
+_PDGID2HERWIGID[-1000015] = 435 ## SSTAU1+
+_PDGID2HERWIGID[ 1000016] = 430 ## SSNUTL
+_PDGID2HERWIGID[-1000016] = 436 ## SSNUTLBR
+_PDGID2HERWIGID[ 2000011] = 437 ## SSEL-
+_PDGID2HERWIGID[-2000011] = 443 ## SSEL+
+_PDGID2HERWIGID[ 2000012] = 438 ## SSNUEL
+_PDGID2HERWIGID[-2000012] = 444 ## SSNUELBR
+_PDGID2HERWIGID[ 2000013] = 439 ## SSMUL-
+_PDGID2HERWIGID[-2000013] = 445 ## SSMUL+
+_PDGID2HERWIGID[ 2000014] = 440 ## SSNUMUL
+_PDGID2HERWIGID[-2000014] = 446 ## SSNUMLBR
+_PDGID2HERWIGID[ 2000015] = 441 ## SSTAU1-
+_PDGID2HERWIGID[-2000015] = 447 ## SSTAU1+
+_PDGID2HERWIGID[ 2000016] = 442 ## SSNUTL
+_PDGID2HERWIGID[-2000016] = 448 ## SSNUTLBR
+_PDGID2HERWIGID[ 1000021] = 449 ## GLUINO
+_PDGID2HERWIGID[ 1000022] = 450 ## NTLINO1
+_PDGID2HERWIGID[ 1000023] = 451 ## NTLINO2
+_PDGID2HERWIGID[ 1000025] = 452 ## NTLINO3
+_PDGID2HERWIGID[ 1000035] = 453 ## NTLINO4
+_PDGID2HERWIGID[ 1000024] = 454 ## CHGINO1+
+_PDGID2HERWIGID[-1000024] = 456 ## CHGINO1-
+_PDGID2HERWIGID[ 1000037] = 455 ## CHGINO2+
+_PDGID2HERWIGID[-1000037] = 457 ## CHGINO2-
+_PDGID2HERWIGID[ 1000039] = 458 ## GRAVTINO
+
+def pdgid2herwigid(pdgid):
+    """
+    Convert a particle ID code in the standard PDG ID code definition into
+    its equivalent in the HERWIG internal IDHW format (as used by ISAWIG).
+    """
+    return _PDGID2HERWIGID.get(pdgid, pdgid)
+
+
+###############################################################################
+## ISAWIG format reading/writing
+
+
 def readISAWIG(isastr, ignorenobr=False):
     """
     Read a spectrum definition from a string in the ISAWIG format, returning
@@ -430,7 +500,6 @@ def readISAWIG(isastr, ignorenobr=False):
     branching ratio of zero.
     """
 
-
     blocks = {}
     decays = {}
     LINES = isastr.splitlines()
@@ -541,127 +610,68 @@ def readISAWIG(isastr, ignorenobr=False):
     MUSS = getnextvalidlineitems()[0]
     blocks["MINPAR"].add_entry((4, MUSS))
     #
+
+    # TODO: Parse RPV boolean and couplings into SLHA2 blocks
+
     return blocks, decays
 
 
-## PDG MC ID codes mapped to HERWIG IDHW codes, based on
-## http://www.hep.phy.cam.ac.uk/~richardn/HERWIG/ISAWIG/susycodes.html
-## + the IDPDG array and section 4.13 of the HERWIG manual.
-_PDGID2HERWIGID = {}
-_PDGID2HERWIGID[      -1] = 7
-_PDGID2HERWIGID[      -2] = 8
-_PDGID2HERWIGID[      -3] = 9
-_PDGID2HERWIGID[      -4] = 10
-_PDGID2HERWIGID[      -5] = 11
-_PDGID2HERWIGID[      -6] = 12
-_PDGID2HERWIGID[      21] = 13
-_PDGID2HERWIGID[      22] = 59
-_PDGID2HERWIGID[      11] = 121
-_PDGID2HERWIGID[      12] = 122
-_PDGID2HERWIGID[      13] = 123
-_PDGID2HERWIGID[      14] = 124
-_PDGID2HERWIGID[      15] = 125
-_PDGID2HERWIGID[      16] = 126
-_PDGID2HERWIGID[     -11] = 127
-_PDGID2HERWIGID[     -12] = 128
-_PDGID2HERWIGID[     -13] = 129
-_PDGID2HERWIGID[     -14] = 130
-_PDGID2HERWIGID[     -15] = 131
-_PDGID2HERWIGID[     -16] = 132
-_PDGID2HERWIGID[      24] = 198 ## W+
-_PDGID2HERWIGID[     -24] = 199 ## W-
-_PDGID2HERWIGID[      23] = 200 ## Z
-_PDGID2HERWIGID[      25] = 203 ## HIGGSL0 (added for PDG standard -> HERWIG IDHW) # TODO: should be 201?
-_PDGID2HERWIGID[      26] = 203 ## HIGGSL0
-_PDGID2HERWIGID[      35] = 204 ## HIGGSH0
-_PDGID2HERWIGID[      36] = 205 ## HIGGSA0
-_PDGID2HERWIGID[      37] = 206 ## HIGGS+
-_PDGID2HERWIGID[     -37] = 207 ## HIGGS-
-_PDGID2HERWIGID[ 1000001] = 401 ## SSDLBR
-_PDGID2HERWIGID[-1000001] = 407 ## SSDLBR
-_PDGID2HERWIGID[ 1000002] = 402 ## SSULBR
-_PDGID2HERWIGID[-1000002] = 408 ## SSUL
-_PDGID2HERWIGID[ 1000003] = 403 ## SSSLBR
-_PDGID2HERWIGID[-1000003] = 409 ## SSSL
-_PDGID2HERWIGID[ 1000004] = 404 ## SSCLBR
-_PDGID2HERWIGID[-1000004] = 410 ## SSCL
-_PDGID2HERWIGID[ 1000005] = 405 ## SSB1BR
-_PDGID2HERWIGID[-1000005] = 411 ## SSB1
-_PDGID2HERWIGID[ 1000006] = 406 ## SST1BR
-_PDGID2HERWIGID[-1000006] = 412 ## SST1
-_PDGID2HERWIGID[ 2000001] = 413 ## SSDR
-_PDGID2HERWIGID[-2000001] = 419 ## SSDRBR
-_PDGID2HERWIGID[ 2000002] = 414 ## SSUR
-_PDGID2HERWIGID[-2000002] = 420 ## SSURBR
-_PDGID2HERWIGID[ 2000003] = 415 ## SSSR
-_PDGID2HERWIGID[-2000003] = 421 ## SSSRBR
-_PDGID2HERWIGID[ 2000004] = 416 ## SSCR
-_PDGID2HERWIGID[-2000004] = 422 ## SSCRBR
-_PDGID2HERWIGID[ 2000005] = 417 ## SSB2
-_PDGID2HERWIGID[-2000005] = 423 ## SSB2BR
-_PDGID2HERWIGID[ 2000006] = 418 ## SST2
-_PDGID2HERWIGID[-2000006] = 424 ## SST2BR
-_PDGID2HERWIGID[ 1000011] = 425 ## SSEL-
-_PDGID2HERWIGID[-1000011] = 431 ## SSEL+
-_PDGID2HERWIGID[ 1000012] = 426 ## SSNUEL
-_PDGID2HERWIGID[-1000012] = 432 ## SSNUELBR
-_PDGID2HERWIGID[ 1000013] = 427 ## SSMUL-
-_PDGID2HERWIGID[-1000013] = 433 ## SSMUL+
-_PDGID2HERWIGID[ 1000014] = 428 ## SSNUMUL
-_PDGID2HERWIGID[-1000014] = 434 ## SSNUMLBR
-_PDGID2HERWIGID[ 1000015] = 429 ## SSTAU1-
-_PDGID2HERWIGID[-1000015] = 435 ## SSTAU1+
-_PDGID2HERWIGID[ 1000016] = 430 ## SSNUTL
-_PDGID2HERWIGID[-1000016] = 436 ## SSNUTLBR
-_PDGID2HERWIGID[ 2000011] = 437 ## SSEL-
-_PDGID2HERWIGID[-2000011] = 443 ## SSEL+
-_PDGID2HERWIGID[ 2000012] = 438 ## SSNUEL
-_PDGID2HERWIGID[-2000012] = 444 ## SSNUELBR
-_PDGID2HERWIGID[ 2000013] = 439 ## SSMUL-
-_PDGID2HERWIGID[-2000013] = 445 ## SSMUL+
-_PDGID2HERWIGID[ 2000014] = 440 ## SSNUMUL
-_PDGID2HERWIGID[-2000014] = 446 ## SSNUMLBR
-_PDGID2HERWIGID[ 2000015] = 441 ## SSTAU1-
-_PDGID2HERWIGID[-2000015] = 447 ## SSTAU1+
-_PDGID2HERWIGID[ 2000016] = 442 ## SSNUTL
-_PDGID2HERWIGID[-2000016] = 448 ## SSNUTLBR
-_PDGID2HERWIGID[ 1000021] = 449 ## GLUINO
-_PDGID2HERWIGID[ 1000022] = 450 ## NTLINO1
-_PDGID2HERWIGID[ 1000023] = 451 ## NTLINO2
-_PDGID2HERWIGID[ 1000025] = 452 ## NTLINO3
-_PDGID2HERWIGID[ 1000035] = 453 ## NTLINO4
-_PDGID2HERWIGID[ 1000024] = 454 ## CHGINO1+
-_PDGID2HERWIGID[-1000024] = 456 ## CHGINO1-
-_PDGID2HERWIGID[ 1000037] = 455 ## CHGINO2+
-_PDGID2HERWIGID[-1000037] = 457 ## CHGINO2-
-_PDGID2HERWIGID[ 1000039] = 458 ## GRAVTINO
-
-def pdgid2herwigid(pdgid):
+def writeISAJET(blocks, decays, outname, ignorenobr=False, precision=8):
     """
-    Convert a particle ID code in the standard PDG ID code definition into
-    its equivalent in the HERWIG internal IDHW format (as used by ISAWIG).
+    Return a SUSY spectrum definition in the format required for input by ISAJET,
+    as a string, from the supplied blocks and decays dicts.
+
+    The outname parameter specifies the desired output filename from ISAJET: this
+    will appear in the first line of the return value.
+
+    If the ignorenobr parameter is True, do not write decay entries with a
+    branching ratio of zero.
     """
-    return _PDGID2HERWIGID.get(pdgid, pdgid)
+    fmte = "%." + str(precision) + "e"
 
+    masses = blocks["MASS"].entries
 
+    ## Init output string
+    out = ""
 
+    ## First line is the output name
+    out += "'%s'" % outname + "\n"
 
-def writeISAWIGFile(isafilename, blocks, decays, **kwargs):
-    """
-    Write an ISAWIG file from the supplied blocks and decays dicts.
+    ## Next the top mass
+    out += fmte % masses[6] + "\n"
 
-    Other keyword parameters are passed to writeISAWIG.
+    ## Next the top mass
+    out += fmte % masses[6] + "\n"
 
-    TODO: Handle RPV SUSY
-    """
-    f = open(isafilename, "w")
-    f.write(writeISAWIG(blocks, decays, kwargs))
-    f.close()
+    ## mSUGRA parameters (one line)
+    # e.g. 1273.78,713.286,804.721,4.82337
+
+    ## Masses and trilinear couplings (3 lines)
+    # e.g. 1163.14,1114.15,1118.99,374.664,209.593
+    # e.g. 1069.54,1112.7,919.908,374.556,209.381,-972.817,-326.745,-406.494
+    # e.g. 1163.14,1114.15,1118.99,374.712,210.328
+
+    ## RPV couplings (?? lines)
+    # e.g. 232.615,445.477
+
+    ## Etc ???!!!
+    # e.g. /
+    # e.g. n
+    # e.g. y
+    # e.g. y
+    # e.g. 0.047441 3.80202e-23 0 0 0 2.17356e-22 0 0 5.23773e-09
+    # e.g. y
+    # e.g. 3.35297e-25 0 0 0 7.34125e-24 0 0 0 3.17951e-22 8.07984e-12 0 0 0 1.76906e-10 0 0 0 7.66184e-09 0 0 0 0 0 0 0 0 0
+    # e.g. n
+    # e.g. 'susy_RPV_stau_BC1scan_m560_tanb05.txt'
+
+    return out
 
 
 def writeISAWIG(blocks, decays, ignorenobr=False, precision=8):
     """
-    Return an ISAWIG definition as a string, from the supplied blocks and decays dicts.
+    Return a SUSY spectrum definition in the format produced by ISAWIG for inut to HERWIG
+    as a string, from the supplied SLHA blocks and decays dicts.
 
     ISAWIG parsing based on the HERWIG SUSY specification format, from
     http://www.hep.phy.cam.ac.uk/~richardn/HERWIG/ISAWIG/file.html
@@ -885,23 +895,96 @@ def writeISAWIG(blocks, decays, ignorenobr=False, precision=8):
     out += (fmte + " " + fmte + " " + fmte + " " + fmte + "\n") % (vmix[1][1], vmix[1][2], vmix[2][1], vmix[2][2])
     umix = blocks["UMIX"].entries
     out += (fmte + " " + fmte + " " + fmte + " " + fmte + "\n") % (umix[1][1], umix[1][2], umix[2][1], umix[2][2])
-    # THETAT,THETAB,THETAL
+    ## THETAT,THETAB,THETAL
     import math
     out += (fmte + " " + fmte + " " + fmte + " " + "\n") % (math.acos(blocks["STOPMIX"].entries[1][1]),
                                                             math.acos(blocks["SBOTMIX"].entries[1][1]),
                                                             math.acos(blocks["STAUMIX"].entries[1][1]))
-    # ATSS,ABSS,ALSS
+    ## ATSS,ABSS,ALSS
     out += (fmte + " " + fmte + " " + fmte + " " + "\n") % (blocks["AU"].entries[3][3],
                                                             blocks["AD"].entries[3][3],
                                                             blocks["AE"].entries[3][3])
-    # MUSS == sign(mu)
+    ## MUSS == sign(mu)
     out += "%f\n" % blocks["MINPAR"].entries[4]
 
-    ## TODO: Handle RPV SUSY
+    ## RPV SUSY
+    isRPV = False
+    out += "%d\n" % isRPV
+    # TODO: Write RPV couplings if RPV is True (lambda1,2,3; 27 params in each, sci format.
+    # TODO: Get the index orderings right
+    # if isRPV: ...
 
     return out
 
 
+###############################################################################
+## File-level functions
+
+
+def readSLHAFile(spcfilename, **kwargs):
+    """
+    Read an SLHA file, returning dictionaries of blocks and decays.
+
+    Other keyword parameters are passed to readSLHA.
+    """
+    f = open(spcfilename, "r")
+    rtn = readSLHA(f.read(), kwargs)
+    f.close()
+    return rtn
+
+
+def writeSLHAFile(spcfilename, blocks, decays, **kwargs):
+    """
+    Write an SLHA file from the supplied blocks and decays dicts.
+
+    Other keyword parameters are passed to writeSLHA.
+    """
+    f = open(spcfilename, "w")
+    f.write(writeSLHA(blocks, decays, kwargs))
+    f.close()
+
+
+def readISAWIGFile(isafilename, **kwargs):
+    """
+    Read a spectrum definition from a file in the ISAWIG format, returning
+    dictionaries of blocks and decays. While this is not an SLHA format, it is
+    informally supported as a useful mechanism for converting ISAWIG spectra to
+    SLHA.
+
+    Other keyword parameters are passed to readSLHA.
+    """
+    f = open(isafilename, "r")
+    rtn = readISAWIG(f.read(), kwargs)
+    f.close()
+    return rtn
+
+
+def writeISAWIGFile(isafilename, blocks, decays, **kwargs):
+    """
+    Write an ISAWIG file from the supplied blocks and decays dicts.
+
+    Other keyword parameters are passed to writeISAWIG.
+    """
+    f = open(isafilename, "w")
+    f.write(writeISAWIG(blocks, decays, kwargs))
+    f.close()
+
+
+def writeISAJETFile(isafilename, blocks, decays, **kwargs):
+    """
+    Write an ISAJET file from the supplied blocks and decays dicts (see writeISAJET).
+
+    Other keyword parameters are passed to writeISAJET.
+    """
+    f = open(isafilename, "w")
+    f.write(writeISAWIG(blocks, decays, kwargs))
+    f.close()
+
+
+
+###############################################################################
+## Main function for module testing
+
 
 if __name__ == "__main__":
     import sys
